@@ -35,7 +35,7 @@ c
          write(*,*)
      *    "#  'Eg(MeV)'  ' /r.l' '/(g/cm2)' '/cm' 'mb/atom'"
          Eg = 0.1d-3            !  100 keV.
-         do while ( Eg .lt. 101.d-3)   ! to 100 MeV
+         do while ( Eg .lt. 101.d-2)   ! to 1GeV
             call epcompp(media, Eg, tprob, p) ! tprob is prob/r.l
             write(*, *) sngl(Eg*1000.d0), 
      *       sngl(tprob), sngl(tprob/media.X0g),
