@@ -5,11 +5,20 @@
 #include  "Ztrackp.h"
       include "Zprivate.h"
 
-      integer i, nev, j, ntp
+      integer i, nev, j, ntp, eof
       record /ptcl/ w(maxn)
 
       call init
       do j = 1, nevent
+         if(inpfileno .gt. 0) then
+            call readinpfile(eof)
+            if(eof .eq. 1) then
+               write(0,*) 
+     *        ' number of events generated is ',j-1
+               goto 100
+            endif
+            call formpjtg(0)
+         endif
          call gencol(w, ntp)
          call cutbyangle(w, ntp, ntp)
          call sortbyke(w, ntp)  ! sort by kinetic energy 
@@ -18,6 +27,9 @@
          endif
          call outresul(w, ntp)
       enddo
+      write(0,*) 
+     *  ' number of events generated is ',nevent
+ 100  continue
       end
 
       subroutine init
@@ -32,14 +44,10 @@
 #include  "Ztrackp.h"
 
       include  "Zprivate.h"
-      character*200 input
+      character*200 input, file
       character*20 uid
-      integer  pjcode, pjsub, pjchg
-      integer  tgcode, tgsub, tgchg
-      integer klena
-      real*8   pjpx, pjpy, pjpz
-      real*8   tgpx, tgpy, tgpz
-      real*8   roots, s
+      integer klena, icon, eof
+
       external cblkManager
       external cblkEvhnp
 
@@ -63,54 +71,38 @@
       call cqUhooki(1, outzero)
 c       make projectile going +z
       call cqUhookc(1, input) 
-      read(input, *) 
-     * pjcode, pjsub, pjchg, pjpx, pjpy, pjpz
-      call cmkptc(pjcode, pjsub, pjchg,   pj)
-
-      if(pjcode .ne. kgnuc) then
-         pjmnum = 1
+      if(input(1:4) .eq. "file") then
+         read(input(5:10), *) inpfileno
+         xyz=index(input, "xyz")
+         call cqUhookc(2, input) 
+         file = ' '
+         file=input(1:klena(input))
+         call copenfw2(inpfileno, file, 1, icon)
+         if(icon .ne. 1) then
+            write(0,*)
+     *      ' file=', file, ' cannot be opened in Gencol'
+            stop 9999
+         endif
+         call readinpfile(eof)
+c          once rewind to read successively for event generation
+         rewind inpfileno
       else
-         pjmnum = pjsub
+         inpfileno=0
+         read(input, *) 
+     *        pjcode, pjsub, pjchg, pjpx, pjpy, pjpz
+         call cqUhookc(2, input) 
+         read(input, *) 
+     *        tgcode, tgsub, tgchg, tgpx, tgpy, tgpz
+         call cqUhookc(3, input)
+         if(input .ne. ' ') then
+            read(input, *) xpos, ypos, zpos
+            xyz = 1
+         else
+            xyz = 0
+         endif
       endif
-      pj.fm.p(1) = pjpx*pjmnum
-      pj.fm.p(2) = pjpy*pjmnum
-      pj.fm.p(3) = pjpz*pjmnum
-      pj.fm.p(4) =
-     * sqrt(pj.fm.p(1)**2 + pj.fm.p(2)**2 + pj.fm.p(3)**2
-     *       + pj.mass**2)
-
-      call cqUhookc(2, input) 
-      read(input, *) 
-     * tgcode, tgsub, tgchg, tgpx, tgpy, tgpz
-c       make taget (rest of moving -z)
-      call cmkptc(tgcode, tgsub, tgchg, tg)
-      if(tgcode .ne. kgnuc) then
-         tgmnum = 1
-      else
-         tgmnum = tgsub
-      endif
-      tg.fm.p(1) = tgpx*tgmnum
-      tg.fm.p(2) = tgpy*tgmnum
-      tg.fm.p(3) = tgpz*tgmnum
-      tg.fm.p(4) =
-     *  sqrt(tg.fm.p(1)**2 + tg.fm.p(2)**2 + tg.fm.p(3)**2 
-     *   +   tg.mass**2)
-c       
-      if(tgpx .eq. 0. .and. tgpy .eq. 0. .and.
-     *    tgpz .eq. 0.)  then
-c          target is at rest; 
-         s= 2*pj.fm.p(4)*tg.mass +tg.mass**2 + pj.mass**2
-      else
-c         by  general formula
-         s = (pj.fm.p(4)+tg.fm.p(4))**2 -
-     *    (pjpx+tgpx)**2 + (pjpy+tgpy)**2+(pjpz+tgpz)**2
-      endif
-      roots = sqrt(s)
-      write(0, *) ' roots/2=', roots/2
-c           get equivalent lab energy; show it as info.
-c           boost to target rest system
-      call cbst1(1, tg, pj, plab)
 
+      call formpjtg(1)
 
       call cfixPrefix('configDummy')
       call csetCosOrEpi('epics')
@@ -135,11 +127,99 @@ c           boost to target rest system
 
       write(0, *) 'Active int. model=',ActiveMdl
       write(0, *) ' equiv. lab E=', plab.fm.p(4)
-      write(*, '(a)') '#  mulsubKEdir  user '
+      if(xyz .eq. 0) then
+         write(*, '(a)') '#  mulsubKEdir  user '
+      else
+         write(*, '(a)') '#  mulsubKExyzdir  user '
+      endif
       write(*, '(a)') '#--------------------------------'
       end
+      subroutine readinpfile(eof)
+      implicit none
+#include "Zptcl.h"      
+      include "Zprivate.h"
+
+      integer eof ! output . data read--> 0
+                  !   eof reached --> 1
+      read(inpfileno,*, end=100)
+     *     pjcode, pjsub, pjchg, pjpx, pjpy, pjpz
+      read(inpfileno,*, end=100)
+     *     tgcode, tgsub, tgchg, tgpx, tgpy, tgpz
+      if(xyz .gt. 0 ) then
+         read(inpfileno,*, end=100) xpos, ypos, zpos
+      endif
+      eof = 0
+      return
+ 100  continue
+      eof = 1
+      end
+c     *******************
+      subroutine formpjtg(confirm)
+c     ******************
+      implicit none
+#include  "Zptcl.h"
+#include  "Zcode.h"
+#include  "Zevhnp.h"
+#include  "Zevhnv.h"
+#include  "Zmass.h"
+#include  "Zmanager.h"
+#include  "Zmanagerp.h"
+#include  "Ztrackp.h"
+
+      include "Zprivate.h"
+
+      integer confirm  !  input. if 0, root s is  not printed.
+                      !         else  printed
+      real*8   roots, s
+c         form projectile  and target
+
+      call cmkptc(pjcode, pjsub, pjchg,   pj)
+         
+      if(pjcode .ne. kgnuc) then
+         pjmnum = 1
+      else
+         pjmnum = pjsub
+      endif
+      pj.fm.p(1) = pjpx*pjmnum
+      pj.fm.p(2) = pjpy*pjmnum
+      pj.fm.p(3) = pjpz*pjmnum
+      pj.fm.p(4) =
+     *     sqrt(pj.fm.p(1)**2 + pj.fm.p(2)**2 + pj.fm.p(3)**2
+     *     + pj.mass**2)
+         
+c       make taget (rest of moving -z)
+      call cmkptc(tgcode, tgsub, tgchg, tg)
+      if(tgcode .ne. kgnuc) then
+         tgmnum = 1
+      else
+         tgmnum = tgsub
+      endif
+      tg.fm.p(1) = tgpx*tgmnum
+      tg.fm.p(2) = tgpy*tgmnum
+      tg.fm.p(3) = tgpz*tgmnum
+      tg.fm.p(4) =
+     *     sqrt(tg.fm.p(1)**2 + tg.fm.p(2)**2 + tg.fm.p(3)**2 
+     *   +   tg.mass**2)
+c       
+      if(tgpx .eq. 0. .and. tgpy .eq. 0. .and.
+     *        tgpz .eq. 0.)  then
+c     target is at rest; 
+         s= 2*pj.fm.p(4)*tg.mass +tg.mass**2 + pj.mass**2
+      else
+c         by  general formula
+         s = (pj.fm.p(4)+tg.fm.p(4))**2 -
+     *    (pjpx+tgpx)**2 + (pjpy+tgpy)**2+(pjpz+tgpz)**2
+      endif
+      roots = sqrt(s)
+      if(confirm .ne. 0) then
+         write(0, *) ' roots/2=', roots/2
+      endif
+cc           boost to target rest system
+      call cbst1(1, tg, pj, plab)
+      end
+c     ************************
       subroutine outresul(a, ntp)
-          implicit none
+      implicit none
 #include  "Zptcl.h"
 #include  "Zcode.h"
 #include  "Zevhnp.h"
@@ -162,9 +242,16 @@ c           boost to target rest system
          wx = a(i).fm.p(1)/p               
          wy = a(i).fm.p(2)/p               
          wz = a(i).fm.p(3)/p               
-         write(*,'(3i3,g14.5,3f17.13, i5)')
+         if(xyz .eq. 0) then
+            write(*,'(3i3,g14.5,3f17.13, i5)')
      *        a(i).code, a(i).subcode, a(i).charge,
      *        a(i).fm.p(4)-a(i).mass, wx, wy, wz, j
+         else
+            write(*,'(3i3,g14.5,1p3E11.3,0p3f17.13, i5)')
+     *        a(i).code, a(i).subcode, a(i).charge,
+     *        a(i).fm.p(4)-a(i).mass, xpos, ypos, zpos, 
+     *        wx, wy, wz, j
+         endif
       enddo
       if(ntp .gt. 0 .or. outzero .eq. 0) then
          write(*, *) 
@@ -265,7 +352,7 @@ c
 c       ke( indx(1) ) is the highest energy
       end
       subroutine outtrace(nev, a, ntp)
-          implicit none
+      implicit none
 #include  "Zptcl.h"
 #include  "Zcode.h"
 #include  "Zevhnp.h"
@@ -280,6 +367,7 @@ c       ke( indx(1) ) is the highest energy
       record /ptcl/ a(ntp)
       integer  i, j, leng, icon, klena
       real*8  p, wx, wy, wz 
+      real  x1, y1, z1, x2, y2, z2
       character*100  tracefile
 
       write(tracefile, *) TraceDir(1:klena(TraceDir))//'/trace', nev
@@ -294,13 +382,25 @@ c       ke( indx(1) ) is the highest energy
      *        +      a(i).fm.p(3)**2 )
          wx = a(i).fm.p(1)/p               
          wy = a(i).fm.p(2)/p               
-         wz = a(i).fm.p(3)/p               
-         write(TraceDev,'(a, i3, g14.4, i3, i2)') 
-     *      '0    0    0 ' ,
+         wz = a(i).fm.p(3)/p 
+         if(xyz .eq. 0) then
+            x1 = 0.
+            y1 = 0.
+            z1 = 0.
+         else
+            x1 = xpos
+            y1 = ypos
+            z1 = zpos
+         endif
+         x2 = x1 + wx*trackl
+         y2 = y1 + wy*trackl
+         z2 = z1 + wz*trackl
+         write(TraceDev,'(3g14.5, i3, g14.4, i3, i2)') 
+     *      x1, y1, z1,
      *      a(i).code,  a(i).fm.p(4) - a(i).mass, a(i).charge,
      *      0
          write(TraceDev, '(3g14.5, i3, g14.4, i3, g14.4)' )
-     *       wx*trackl, wy*trackl, wz*trackl,
+     *      x2, y2, z2,
      *      a(i).code,  a(i).fm.p(4) - a(i).mass, a(i).charge,
      *      trackl 
          write(TraceDev, *) 
