@@ -21,6 +21,8 @@ c                      -1-->not cross
 c           compute the minimum length to the boundary
       if(comp.struc(1:8) .eq. 'angle') then
          call epbangle(comp, posl, dirl,  length, icon)
+      elseif(comp.struc(1:8) .eq. 'sqpipe') then
+         call epbsqpipe(comp, posl, dirl, length, icon)
 c---  elseif(comp.struc(1:8) .eq. '%2') then
 c---     call epb%2(comp, posl, dirl,  length, icon)
       else
@@ -45,6 +47,8 @@ c             see if pos is inside the comp component
 
       if(comp.struc(1:8) .eq. 'angle')  then
          call epsangle(comp, pos, icon)
+      elseif(comp.struc(1:8) .eq. 'sqpipe') then
+         call epssqpipe(comp, pos, icon)
 c---  elseif(comp.struc(1:8) .eq. '%2')  then
 c---     call eps%2(comp, pos, icon)
       else
@@ -66,6 +70,8 @@ c            read configuration data
 
       if(shape(1:8)  .eq. 'angle') then
          call eprangle(comp)
+      elseif(shape(1:8) .eq. 'sqpipe') then
+         call eprsqpipe(comp)
 c---  elseif(shape(1:8)  .eq. '%2') then
 c---     call epr%2(comp)
       else
@@ -90,6 +96,8 @@ c
 
       if(comp.struc(1:8) .eq. 'angle') then
          call epenvlpangle(comp, orig, abc)
+      elseif(comp.struc(1:8) .eq. 'sqpipe') then
+         call epenvlpsqpipe(comp, orig, abc)
 c---  elseif(comp.struc(1:8) .eq. '%2') then
 c---     call epenvlp%2(comp, orig, abc)
       else
@@ -112,6 +120,8 @@ c
                          !  loc size must be >= comp.Nattributes
       if(comp.struc(1:8) .eq. 'angle') then
          call epatlocangle(comp, loc)
+      elseif(comp.struc(1:8) .eq. 'sqpipe') then
+         call epatlocsqpipe(comp, loc)
 c---  elseif(comp.struc(1:8) .eq. '%2') then
 c---     call epatloc%2(comp, loc)
       else
