@@ -832,29 +832,45 @@ c      ****************************************
        subroutine epprog
        implicit none
 c
-c     4     processes, i.e, pair creation, compton, Photo-electric
-c     effect and Photo-production of pions are considered.
+c     5     processes, i.e, pair creation, compton, Photo-electric
+c     effect, coherent scatt.
+c      and Photo-production of pions are considered.
 c
 #include  "ZepTrackv.h"
 #include  "ZepTrackp.h"
 
 c
-      real*8 tcomp, tphot, tpair, tgp, t
-      real*8 E, prob
+      real*8 tcomp, tphot, tpair, tgp, tcoh,  t
+      real*8 E, prob, xprob(3), txray(3)
       real*8  pairmfp, dl, dx, tmpair
+      real*4  xsec(3)  !  coh, incoh,  P.E  1/(g/cm2)
+      integer icon
 
       E = cTrack.p.fm.p(4)
       if(E .le. EupperBndCS) then
-         call epcompp(Media(MediaNo), E, prob, tcomp)
+         xprob(3)= 0.
+         if( Media(MediaNo).xcom.size .gt. 0 .and.
+     *        E .lt. 1.0e-3 ) then
+c               below 1MeV, use accurate xs.
+            call epXrayp(Media(MediaNo), E,  3,  xprob, txray)
+            tcomp=txray(2)
+            tcoh = txray(1)
+            tphot = txray(3)
+         else
+            tcoh = 1.e35
+            call epcompp(Media(MediaNo), E, prob, tcomp)
+         endif
          if(Photo) then
-c            call epphotoEp(Media(MediaNo).pe,  E, prob, tphot)
-            call epphotoEp(Media(MediaNo),  E, prob, tphot)  ! v8.0
+            if(xprob(3)  .eq. 0.) then
+               call epphotoEp(Media(MediaNo),  E, prob, tphot) ! v8.0
+            endif
          else
             tphot = 1.e35
          endif
       else
          tcomp =1.e35
          tphot= 1.e35
+         tcoh = 1.e35
       endif
       if(E .gt. ElowerBndPair) then
          call epPrSampP(Media(MediaNo), E, prob, tpair)
@@ -875,20 +891,21 @@ c         call ep2cosCond(Media(MediaNo).Aeff, Media(MediaNo).Zeff)
       endif
 
       if(MagPair .eq. 1) then
-
          call epmpairp(cTrack.p, Bfield, Xai, pairmfp, dl)
          dx = dl / Media(MediaNo).gtocm
          tmpair = dx / Media(MediaNo).X0g
       else
          tmpair = 1.e35
       endif
-      t=min(tpair, tcomp, tphot, tgp, tmpair)
+      t=min(tpair, tcomp, tphot, tgp, tmpair, tcoh)
       if(t .eq. tpair) then
          Move.proc='pair'
       elseif(t .eq. tcomp) then
          Move.proc='comp'
       elseif(t .eq. tphot) then
          Move.proc='phot'
+      elseif(t .eq. tcoh) then
+         Move.proc='coh'
       elseif(t .eq. tgp) then
          Move.proc='photop'
       else
@@ -1321,6 +1338,8 @@ c
             call eppair
          elseif(Move.proc .eq. 'phot') then
             call epphot
+         elseif(Move.proc .eq. 'coh') then
+            call epcoher
          elseif(Move.proc .eq. 'photop') then
 c
             call ep2cosPtcl( cTrack.p )
@@ -1381,6 +1400,34 @@ c               for heavey incident.
       endif
       icon = 1
       end
+c     ******************
+      subroutine epcoher
+c        coherent scattering
+c           since coherent scattering is effective at
+c           low energies where angular distribution can be
+c           approximated by (1+cos^2) dcos, we simply use
+       implicit none
+#include  "ZepTrackv.h"
+c
+       record /epDirec/ w
+
+       real*8  eg, tmp, cosg
+       real*8  cs, sn, sing
+
+c             sample scattering angle from (1+cos^2)dcos
+       call ksampRSA(cosg)
+       tmp=1.d0-cosg*cosg
+       sing = sqrt(tmp)
+       call kcossn(cs,sn)
+
+       w.x = cs*sing
+       w.y = sn*sing
+       w.z = cosg
+       call eptransVect(cTrack.w,  w, cTrack.w)
+c        energy unchaged;  
+       call epe2p(cTrack)
+       call eppush(cTrack)
+       end
 c     ******************
       subroutine epcmpt
        implicit none
@@ -1539,13 +1586,14 @@ c      ************
 #include  "Zcnfig.h"
 #include  "Zmass.h"           
 
-       real*8 eout
+       real*8 eout, cost, cs, sn, sint
        logical kbtest
+       record /epTrack/ elec1
 c
 c           get Photo-electron energy
 c       call epphotoEe(Media(MediaNo).pe,   < v8.0
        call epphotoEe(Media(MediaNo),
-     *      cTrack.p.fm.p(4), eout)
+     *      cTrack.p.fm.p(4), eout, cost)
        if(kbtest(Eabsorb, BitPhotoElec)) then
 c            energy absorbed by atom is Eabs = Eshell= Eg-(Ee-Me)
           Move.dE = cTrack.p.fm.p(4) - (eout - masele)
@@ -1557,8 +1605,16 @@ c                  regard it as deposited in the media
              call userde(1, cTrack, Move, Media(MediaNo))
           endif
        endif
+
+       call kcossn(cs,sn)
+       sint = sqrt(1.-cost**2)
+       elec1.w.x = cs*sint
+       elec1.w.y = sn*sint
+       elec1.w.z = cost
        cTrack.p.fm.p(4) = eout 
+       call eptransVect(cTrack.w, elec1.w, cTrack.w)
        call cmkptc(kelec, regptcl, -1, cTrack.p)
+       call epe2p(cTrack)
        call eppush(cTrack)
 ccc       if(Move.Abort .ne. 0) call epempty
        end
